# -*- coding: utf-8 -*-
# Copyright (c) 2003-2006 Ingeniweb SAS
#
# This software is subject to the provisions of the GNU General Public
# License, Version 2.0 (GPL).  A copy of the GPL should accompany this
# distribution.  THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL
# EXPRESS OR IMPLIED WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF TITLE, MERCHANTABILITY,
# AGAINST INFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE
#
# More details in the ``LICENSE`` file included in this package.
"""
Our main test case
$Id: test_zwikifolder.py 35065 2006-12-26 15:51:45Z glenfant $
"""

__author__  = 'Gilles Lenfant <gilles.lenfant@ingeniweb.com>'
__docformat__ = 'restructuredtext'

from Products.CMFCore.utils import getToolByName
from Products.ZWikiFolder.config import *

from base import BaseTestCase


class FakeRequest:
    
    form = {}


class ZWikiFolderTest(BaseTestCase):
    """All our tests"""

    def afterSetUp(self):
        """
        Building our test data
        """
        self.loginAsPortalOwner()
        self.portal.invokeFactory('ZWikiFolder', 'zfolder')
        self.zfolder = getattr(self.portal, 'zfolder')
        self.logout()
        return

    
    def test_defaultContent(self):
        """
        Do we have the FrontPage?
        """
        
        self.failUnless('FrontPage' in self.zfolder.objectIds())
        return


    def test_vocabMailingPage(self):
        """
        Do we see all pages?
        """

        vocab = self.zfolder.vocabMailingPage()
        self.assertEqual(vocab.items(), (('FrontPage', 'FrontPage'),))
        return

    def test_mailSignature(self):
        """
        Do we manage the 'mail_signature' property correctly?
        """

        # There is no 'mail_signature' prop in newly created ZWikiFolder
        self.failUnless(not self.zfolder.hasProperty('mail_signature'))

        # Check we have a new signature
        self.zfolder.setMailSignature(" blah ")
        self.failUnlessEqual(self.zfolder.getMailSignature(), 'blah')
        self.failUnlessEqual(self.zfolder.getProperty('mail_signature'), 'blah')

        # Check we have no custom signature again
        self.zfolder.setMailSignature("    ")
        self.failUnlessEqual(self.zfolder.getMailSignature(), '')
        self.failUnless(not self.zfolder.hasProperty('mail_signature'))
        return


    def test_listOfRoles(self):
        """
        Do we pick all roles except Manager?
        """

        expected = (
            (PSEUDO_ACQUIRED_ROLE, 'Acquired roles'),
            ('Anonymous', 'Anonymous'),
            ('Authenticated', 'Authenticated'),
            ('Member', 'Member'),
            ('Owner', 'Owner'),
            ('Reviewer', 'Reviewer'))
        rl = self.zfolder.listOfRoles()
        self.assertEqual(rl.items(), expected)
        return


    def test_permissionAccessorMutator(self):
        """
        Do we manage permissions correctly?
        """
        
        field_names = (
            'add_page_permission',
            'add_comment_permission',
            'change_page_type_permission',
            'delete_page_permission',
            'edit_page_permission',
            'rename_page_permission',
            'rate_page_permission',
            'reparent_page_permission')
        
        for field_name in field_names:
            permission = self.zfolder._fieldname_to_perm[field_name]
            field = self.zfolder.Schema()[field_name]

            # Check default roles
            roles = self.zfolder.permissionAccessor(field=field)
            self.assertEqual(roles, [PSEUDO_ACQUIRED_ROLE, 'Manager'])
            
            # Check adding role
            roles.append('Authenticated')
            self.zfolder.permissionMutator(roles, field=field_name)
            self.assertEqual(roles, [PSEUDO_ACQUIRED_ROLE, 'Manager', 'Authenticated'])
            
            # Check Manager is always granted whatever the user selects
            self.zfolder.permissionMutator(['Authenticated'], field=field_name)
            roles = self.zfolder.permissionAccessor(field=field)
            self.assertEqual(roles, ['Authenticated', 'Manager'])
        return


    def test_post_validate(self):
        """
        Do we validate the form correctly?
        """
        
        REQUEST = FakeRequest()
        
        # Invalid user or mail address
        REQUEST.form = {'subscriber_list': ['nosuchuser']}
        errors = {}
        self.zfolder.post_validate(REQUEST=REQUEST, errors=errors)
        self.assertEqual(errors.has_key('subscriber_list'), True)
        
        # Valid mail addresses
        REQUEST.form = {'subscriber_list': ['somebody@somewhere.com', 'otherguy@elsewhere.net:edits']}
        errors = {}
        self.zfolder.post_validate(REQUEST=REQUEST, errors=errors)
        self.assertEqual(errors.has_key('subscriber_list'), False)
        
        # Valid member ids
        portal_membership = getToolByName(self.portal, 'portal_membership')
        member_ids = portal_membership.listMemberIds()
        REQUEST.form = {'subscriber_list': member_ids}
        errors = {}
        self.zfolder.post_validate(REQUEST=REQUEST, errors=errors)
        self.assertEqual(errors.has_key('subscriber_list'), False)
        
        # Valid member ids with 'edits'
        member_ids = [mid + ':edits' for mid in member_ids]
        REQUEST.form = {'subscriber_list': member_ids}
        errors = {}
        self.zfolder.post_validate(REQUEST=REQUEST, errors=errors)
        self.assertEqual(errors.has_key('subscriber_list'), False)
        return


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(ZWikiFolderTest))
    return suite
