# -*- coding: utf-8 -*-
# Copyright (c) 2003-2006 Ingeniweb SAS
#
# This software is subject to the provisions of the GNU General Public
# License, Version 2.0 (GPL).  A copy of the GPL should accompany this
# distribution.  THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL
# EXPRESS OR IMPLIED WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF TITLE, MERCHANTABILITY,
# AGAINST INFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE
#
# More details in the ``LICENSE`` file included in this package.
"""
The ZWikiFolder content type
$Id: zwikifolder.py 35307 2007-01-03 18:15:35Z glenfant $
"""

__author__  = 'Gilles Lenfant <gilles.lenfant@ingeniweb.com>'
__docformat__ = 'restructuredtext'

from AccessControl import ClassSecurityInfo
from AccessControl import Permissions as zope_permissions
from AccessControl.SecurityManagement import newSecurityManager
from AccessControl.SecurityManagement import getSecurityManager
from AccessControl import SpecialUsers
from AccessControl.PermissionRole import rolesForPermissionOn

from Products.CMFCore import permissions as cmf_permissions
from Products.CMFCore.utils import getToolByName
from Products.Archetypes import public as at_api
from Products.ATContentTypes.content.folder import ATBTreeFolder
from Products.ATContentTypes.content.folder import ATBTreeFolderSchema
from Products.ZWiki import Permissions as zwiki_permissions

from config import *

schema = ATBTreeFolderSchema.copy() + at_api.Schema((
    # Default schemata
    at_api.StringField(
        'allowed_page_types',
        multiValued=True,
        default=DEFAULT_ALLOWED_PAGE_TYPES,
        vocabulary=at_api.DisplayList(ALLOWED_PAGE_TYPES),
        enforceVocabulary=True,
        widget=at_api.InAndOutWidget(
            label="Allowed page types",
            description="Select wiki page markup types the authors may add, the first type is the default one for new wiki pages.",
            label_msgid='label_allowed_page_types',
            description_msgid='help_allowed_page_types',
            i18n_domain=I18N_DOMAIN)
        ),

    at_api.BooleanField(
        'use_wikiname_links',
        default=True,
        widget=at_api.BooleanWidget(
            label="Enable CamelCase links?",
            description=("Words starting with uppercase and mixing lowercase and uppercase are wiki links. "
                         "<strong>IamAWikiLink</strong>"),
            label_msgid='label_use_wikiname_links',
            description_msgid='help_use_wikiname_links',
            i18n_domain=I18N_DOMAIN)
        ),

    at_api.BooleanField(
        'use_bracket_links',
        default=True,
        widget=at_api.BooleanWidget(
            label="Enable [single bracket] links?",
            description=("Groups of words enclosed within matching single brackets are wiki links. "
                         "<strong>[I am a wiki link]</strong>"),
            label_msgid='label_use_bracket_links',
            description_msgid='help_use_bracket_links',
            i18n_domain=I18N_DOMAIN)
        ),

    at_api.BooleanField(
        'use_double_bracket_links',
        default=True,
        widget=at_api.BooleanWidget(
            label="Enable [[double bracket]] links?",
            description=("Groups of words enclosed within matching double brackets are wiki links (Wikipedia style). "
                         "<strong>[[I am a wiki link]]</strong>"),
            label_msgid='label_use_double_bracket_links',
            description_msgid='help_use_double_bracket_links',
            i18n_domain=I18N_DOMAIN)
        ),

    at_api.BooleanField(
        'show_subtopics',
        default=False,
        widget=at_api.BooleanWidget(
            label="Show subtopics",
            description="Show links to children pages at the end of each page.",
            label_msgid='label_show_subtopics',
            description_msgid='help_show_subtopics',
            i18n_domain=I18N_DOMAIN)
        ),
    
    # Mailing schemata
    at_api.StringField(
        'mail_from',
        schemata='Mailing',
        validators=('isEmail',),
        widget=at_api.StringWidget(
            label="Mail \"From\" address",
            description=("Sets the mailout \"From\" address. <strong>Page subscription is disabled unless"
                         " you provide a valid mail address here</strong>"),
            label_msgid='label_mail_from',
            description_msgid='help_mail_from',
            i18n_domain=I18N_DOMAIN,
            size=MAIL_WIDGET_SIZE)
        ),

    at_api.StringField(
        'mail_replyto',
        schemata='Mailing',
        validators=('isEmail',),
        widget=at_api.StringWidget(
            label="Mail \"Reply-to\" address",
            description="Sets the mailout \"Reply-to\" address. If unspecified, above \"From\" address is used.",
            label_msgid='label_mail_replyto',
            description_msgid='help_mail_replyto',
            i18n_domain=I18N_DOMAIN,
            size=MAIL_WIDGET_SIZE)
        ),

    at_api.StringField(
        'mail_to',
        schemata='Mailing',
        validators=('isEmail',),
        widget=at_api.StringWidget(
            label="Mail \"To\" address",
            description="Sets the mailout \"To\" address. If unspecified, above \"Reply-to\" address is used.",
            label_msgid='label_mail_to',
            description_msgid='help_mail_to',
            i18n_domain=I18N_DOMAIN,
            size=MAIL_WIDGET_SIZE)
        ),

    at_api.StringField(
        'mail_admin',
        schemata='Mailing',
        validators=('isEmail',),
        widget=at_api.StringWidget(
            label="Mail admin address",
            description="Recipient of errors raised when mailing notifications.",
            label_msgid='label_mail_admin',
            description_msgid='help_mail_admin',
            i18n_domain=I18N_DOMAIN,
            size=MAIL_WIDGET_SIZE)
        ),

    at_api.BooleanField(
        'auto_subscribe',
        schemata='Mailing',
        widget=at_api.BooleanWidget(
            label="Auto subscribe?",
            description="If checked, authors and commenters of wiki pages will be notified.",
            label_msgid='label_auto_subscribe',
            description_msgid='help_auto_subscribe',
            i18n_domain=I18N_DOMAIN)
        ),

    at_api.StringField(
        'subscriber_list',
        schemata='Mailing',
        widget=at_api.LinesWidget(
            label="Subscribers to all pages of this ZWikiFolder",
            description=("Mail addresses and user names are enabled - one per line. "
                         "Names and addresses may be suffixed by \":edits\" to receive mails on changes."),
            label_msgid='label_subscriber_list',
            description_msgid='help_subscriber_list',
            i18n_domain=I18N_DOMAIN)
    ),
    
    at_api.StringField(
        'default_mailing_page',
        schemata='Mailing',
        vocabulary="vocabMailingPage",
        enforceVocabulary=True,
        accessor='_get_default_mailing_page',
        widget=at_api.SelectionWidget(
            label="Default mailing page",
            description="The name of the default destination page for mailins with no page name in the subject.",
            label_msgid='label_default_mailing_page',
            description_msgid='help_default_mailing_page',
            i18n_domain=I18N_DOMAIN)
        ),

    at_api.StringField(
        'mail_subject_prefix',
        schemata='Mailing',
        widget=at_api.StringWidget(
            label="Mail subject prefix",
            description="All outgoing mails will be prefixed with this text. Example: \"[ThisWiki]\".",
            label_msgid='label_mail_subject_prefix',
            description_msgid='help_mail_subject_prefix',
            i18n_domain=I18N_DOMAIN)
        ),
    
    at_api.BooleanField(
        'mail_page_name',
        schemata='Mailing',
        default=True,
        widget=at_api.BooleanWidget(
            label="Include page name in the mail subject?",
            description="Do we put the page name in the mail subject?",
            label_msgid='label_mail_page_name',
            description_msgid='help_mail_page_name',
            i18n_domain=I18N_DOMAIN)
        ),
    
    at_api.TextField(
        'mail_signature_field', # '...field' because we do not want duplicate attr name with property 'mail_signature'
        schemata='Mailing',
        edit_accessor='getMailSignature',
        mutator='setMailSignature',
        widget=at_api.TextAreaWidget(
            label="Signature appended to mails",
            description=("This text will be appended to all outgoing mails. "
                         "<strong>Warning:</strong> Adding a custom signature will remove the page URL from mails."),
            label_msgid='label_mail_signature',
            description_msgid='help_mail_signature',
            i18n_domain=I18N_DOMAIN)
        ),
    
    at_api.StringField(
        'mailout_policy',
        schemata='Mailing',
        vocabulary=at_api.DisplayList(
            [('comments', "Mail out on comments only", 'label_mailout_policy_comments'),
             ('edits', "Mail out on comments, changes, and new pages", 'label_mailout_policy_edits')]),
        enforceVocabulary=True,
        default='comments',
        widget=at_api.SelectionWidget(
            label="Mail out policy",
            description="Mails will be sent on changes of wiki pages depending on the chosen policy",
            label_msgid='label_mailout_policy',
            description_msgid='help_mailout_policy',
            i18n_domain=I18N_DOMAIN)
        ),
    
    # Security related fields

    at_api.StringField(
        'add_page_permission',
        multiValued=True,
        schemata='Security',
        vocabulary='listOfRoles',
        enforceVocabulary=True,
        default=[PSEUDO_ACQUIRED_ROLE],
        accessor='add_page_permissionAccessor',
        mutator='add_page_permissionMutator',
        write_permission=zope_permissions.change_permissions,
        widget=at_api.MultiSelectionWidget(
            macro='zwikifolder_permission_widget',
            label="Roles to add wiki pages",
            description=("Please select the roles granted to add wiki pages here."
                         " Do not change this unless you know what roles and permission mean."),
            label_msgid='label_add_page_permission',
            description_msgid='help_add_page_permission',
            i18n_domain=I18N_DOMAIN)
        ),

    at_api.StringField(
        'add_comment_permission',
        multiValued=True,
        schemata='Security',
        vocabulary='listOfRoles',
        enforceVocabulary=True,
        default=[PSEUDO_ACQUIRED_ROLE],
        accessor='add_comment_permissionAccessor',
        mutator='add_comment_permissionMutator',
        write_permission=zope_permissions.change_permissions,
        widget=at_api.MultiSelectionWidget(
            macro='zwikifolder_permission_widget',
            label="Roles to comment wiki pages",
            description=("Please select the roles granted to add comments to wiki pages."
                         " Do not change this unless you know what roles and permission mean."),
            label_msgid='label_add_comment_permission',
            description_msgid='help_add_comment_permission',
            i18n_domain=I18N_DOMAIN)
        ),
    
    at_api.StringField(
        'change_page_type_permission',
        multiValued=True,
        schemata='Security',
        vocabulary='listOfRoles',
        enforceVocabulary=True,
        default=[PSEUDO_ACQUIRED_ROLE],
        accessor='change_page_type_permissionAccessor',
        mutator='change_page_type_permissionMutator',
        write_permission=zope_permissions.change_permissions,
        widget=at_api.MultiSelectionWidget(
            macro='zwikifolder_permission_widget',
            label="Roles to change type of wiki pages",
            description=("Please select the roles granted to change type of wiki pages. I.e.: StructuredText to reStructuredText."
                         " Do not change this unless you know what roles and permission mean."),
            label_msgid='label_change_page_type_permission',
            description_msgid='help_change_page_type_permission',
            i18n_domain=I18N_DOMAIN)
        ),
    
    at_api.StringField(
        'delete_page_permission',
        multiValued=True,
        schemata='Security',
        vocabulary='listOfRoles',
        enforceVocabulary=True,
        default=[PSEUDO_ACQUIRED_ROLE],
        accessor='delete_page_permissionAccessor',
        mutator='delete_page_permissionMutator',
        write_permission=zope_permissions.change_permissions,
        widget=at_api.MultiSelectionWidget(
            macro='zwikifolder_permission_widget',
            label="Roles to delete wiki pages",
            description=("Please select the roles granted to delete wiki pages."
                         " Do not change this unless you know what roles and permission mean."),
            label_msgid='label_delete_page_permission',
            description_msgid='help_delete_page_permission',
            i18n_domain=I18N_DOMAIN)
        ),
    
    at_api.StringField(
        'edit_page_permission',
        multiValued=True,
        schemata='Security',
        vocabulary='listOfRoles',
        enforceVocabulary=True,
        default=[PSEUDO_ACQUIRED_ROLE],
        accessor='edit_page_permissionAccessor',
        mutator='edit_page_permissionMutator',
        write_permission=zope_permissions.change_permissions,
        widget=at_api.MultiSelectionWidget(
            macro='zwikifolder_permission_widget',
            label="Roles to change wiki pages",
            description=("Please select the roles granted to change existing wiki pages."
                         " Do not change this unless you know what roles and permission mean."),
            label_msgid='label_edit_page_permission',
            description_msgid='help_edit_page_permission',
            i18n_domain=I18N_DOMAIN)
        ),

    at_api.StringField(
        'rename_page_permission',
        multiValued=True,
        schemata='Security',
        vocabulary='listOfRoles',
        enforceVocabulary=True,
        default=[PSEUDO_ACQUIRED_ROLE],
        accessor='rename_page_permissionAccessor',
        mutator='rename_page_permissionMutator',
        write_permission=zope_permissions.change_permissions,
        widget=at_api.MultiSelectionWidget(
            macro='zwikifolder_permission_widget',
            label="Roles to rename wiki pages",
            description=("Please select the roles granted to change names of existing wiki pages."
                         " Do not change this unless you know what roles and permission mean."),
            label_msgid='label_rename_page_permission',
            description_msgid='help_rename_page_permission',
            i18n_domain=I18N_DOMAIN)
        ),

    at_api.StringField(
        'rate_page_permission',
        multiValued=True,
        schemata='Security',
        vocabulary='listOfRoles',
        enforceVocabulary=True,
        default=[PSEUDO_ACQUIRED_ROLE],
        accessor='rate_page_permissionAccessor',
        mutator='rate_page_permissionMutator',
        write_permission=zope_permissions.change_permissions,
        widget=at_api.MultiSelectionWidget(
            macro='zwikifolder_permission_widget',
            label="Roles to rate wiki pages",
            description=("Please select the roles granted to rate existing wiki pages."
                         " Do not change this unless you know what roles and permission mean."),
            label_msgid='label_rate_page_permission',
            description_msgid='help_rate_page_permission',
            i18n_domain=I18N_DOMAIN)
        ),

    at_api.StringField(
        'reparent_page_permission',
        multiValued=True,
        schemata='Security',
        vocabulary='listOfRoles',
        enforceVocabulary=True,
        default=[PSEUDO_ACQUIRED_ROLE],
        accessor='reparent_page_permissionAccessor',
        mutator='reparent_page_permissionMutator',
        write_permission=zope_permissions.change_permissions,
        widget=at_api.MultiSelectionWidget(
            macro='zwikifolder_permission_widget',
            label="Roles to reparent wiki pages",
            description=("Please select the roles granted to change the parent of existing wiki pages."
                         " Do not change this unless you know what roles and permission mean."),
            label_msgid='label_reparent_page_permission',
            description_msgid='help_reparent_page_permission',
            i18n_domain=I18N_DOMAIN)
        ),
))


class ZWikiFolder(ATBTreeFolder):
    """A folder dedicated to ZWiki Pages"""

    schema = schema
    content_icon = 'zwikifolder_icon.gif'
    meta_type = PROJECTNAME
    portal_type = PROJECTNAME
    archetype_name = PROJECTNAME
    immediate_view = 'folder_listing'
    default_view = 'folder_listing'
    suppl_views = ('folder_summary_view', 'folder_tabular_view', 'atct_album_view')
    global_allow = True
    typeDescription = __doc__
    typeDescMsgId = 'description_edit_zwikifolder'
    assocMimetypes = ()
    assocFileExt = ()
    cmf_edit_kws = ()

    security = ClassSecurityInfo()

    # About these properties: http://zwiki.org/QuickReference
    # Note that we change 'mail_signature' property programmatically
    # in 'setMailSignature' mutator
    _properties = ATBTreeFolder._properties + (
        {'id': 'allowed_page_types',
         'type': 'lines',
         'mode': 'w'
         },
        {'id': 'use_wikiname_links',
         'type': 'boolean',
         'mode': 'w'
         },
        {'id': 'use_bracket_links',
         'type': 'boolean',
         'mode': 'w'
         },
        {'id': 'use_double_bracket_links',
         'type': 'boolean',
         'mode': 'w'
         },
        {'id': 'show_subtopics',
         'type': 'boolean',
         'mode': 'w'
         },
        {'id': 'mail_admin',
         'type': 'string',
         'mode': 'w'
         },
        {'id': 'mail_from',
         'type': 'string',
         'mode': 'w'
         },
        {'id': 'mail_replyto',
         'type': 'string',
         'mode': 'w'
         },
        {'id': 'mail_to',
         'type': 'string',
         'mode': 'w'
         },
        {'id': 'auto_subscribe',
         'type': 'boolean',
         'mode': 'w'
         },
        {'id': 'subscriber_list',
         'type': 'lines',
         'mode': 'w'
         },
        {'id': 'default_mailing_page',
         'type': 'string',
         'mode': 'w'
         },
        {'id': 'mail_subject_prefix',
         'type': 'string',
         'mode': 'w'
         },
        {'id': 'mail_page_name',
         'type': 'boolean',
         'mode': 'w'
         },
        {'id': 'mailout_policy',
         'type': 'string',
         'mode': 'w'
         }
    )
    
    # Default properties values
    allowed_page_types = DEFAULT_ALLOWED_PAGE_TYPES
    use_wikiname_links = True
    use_bracket_links = True
    use_double_bracket_links = True
    show_subtopics = False
    auto_subscribe = False
    subscriber_list = []
    mail_page_name = True
    mailout_policy = 'comments'


    security.declareProtected(cmf_permissions.ModifyPortalContent, 'initializeArchetype')
    def initializeArchetype(self, **kwargs):
        """
        Create default ZWiki page
        @param kwargs: default edit params
        """

        try:
            super(ZWikiFolder, self).initializeArchetype(**kwargs)
        except TypeError, e:
            # Uh we're certainly in a Zope 2.7.x (ExtensioClass is not "new style")
            ZWikiFolder.inheritedAttribute('initializeArchetype')(self, **kwargs)
        
        # We'll ignore security settings on creating new ZWiki Page because we
        # consider that if an user is granted to add a ZWiki Folder, he should
        # be allowed to add a ZWiki Page too. So we "sudo" the creation of the
        # default FrontPage
        actual_user = getSecurityManager().getUser()
        newSecurityManager(self.REQUEST, SpecialUsers.system)
        self.invokeFactory('Wiki Page', 'FrontPage', page_type=DEFAULT_ALLOWED_PAGE_TYPES[0], file=DEFAULT_FRONTPAGE)
        newSecurityManager(self.REQUEST, actual_user)
        
        # Owner of ZWikiFolder can change the security policy
        self.manage_permission(zope_permissions.change_permissions, roles=['Owner'])
        return

    ###
    ## Vocabulary methods
    ###
    
    def vocabMailingPage(self):
        """
        Possible pages for default_mailng_pages
        @return: Displayist of wiki pages
        """
        
        wiki_pages = self.objectItems(spec='ZWiki Page')
        return at_api.DisplayList([(oid, page.title_or_id()) for oid, page in wiki_pages])
    

    def listOfRoles(self):
        """
        Possible roles to be granted to do stuffs here, Manager excluded
        @return: DisplayList of valid roles
        """
        
        valid_roles = [(PSEUDO_ACQUIRED_ROLE, 'Acquired roles', 'label_acquired_roles')]
        valid_roles.extend([(r, r, r) for r in self.validRoles() if r != 'Manager'])
        return at_api.DisplayList(valid_roles)

    ###
    ## Accessors & mutators
    ###

    def getMailSignature(self, **kwargs):
        """
        Accessor for 'mail_signature_field'
        @param kwargs: See Products/Archetypes/Field.py
        """

        return self.getProperty('mail_signature', d='')
            


    def setMailSignature(self, signature, **kwargs):
        """
        Mutator for 'mail_signature_field' field
        Stores on property 'mail_signature' if not empty, else deletes this property
        See how ZWiki uses 'mail_signature' to understand why this strange way to do
        @param signature: the new signature text
        @param kwargs: See Products/Archetypes/Field.py
        """

        signature = signature.strip()
        if signature:
            
            if self.hasProperty('mail_signature'):
                self.signature = signature
            else:
                self._setProperty('mail_signature', signature, type='text')
        else:
            if self.hasProperty('mail_signature'):
                self._delProperty('mail_signature')
        return


    _fieldname_to_perm = {
        # {field name: permission, ...}
        'add_page_permission': zwiki_permissions.AddPage,
        'add_comment_permission': zwiki_permissions.Comment,
        'change_page_type_permission': zwiki_permissions.ChangeType,
        'delete_page_permission': zwiki_permissions.Delete,
        'edit_page_permission': zwiki_permissions.Edit,
        'rename_page_permission': zwiki_permissions.Rename,
        'rate_page_permission': zwiki_permissions.Rate,
        'reparent_page_permission': zwiki_permissions.Reparent
    }

    security.declarePrivate('permissionAccessor')
    def permissionAccessor(self, **kwargs):
        """
        Provides roles for a permission
        @param kwargs: See Products/Archetypes/Field.py
        @return: [role, ...] for which permission associated to field is granted
        """
        
        field_name = kwargs['field'].getName()
        permission = self._fieldname_to_perm.get(field_name, None)
        roles = []
        if permission:
            if self.acquiredRolesAreUsedBy(permission):
                roles = [PSEUDO_ACQUIRED_ROLE]
            roles_maps = self.rolesOfPermission(permission)
            roles.extend([rm['name'] for rm in roles_maps if rm['selected'] == 'SELECTED'])
        return roles

    # We must alias permissionAccessor: AT screams when same accessor is shared by 2+ fields
    add_page_permissionAccessor = permissionAccessor
    add_comment_permissionAccessor = permissionAccessor
    change_page_type_permissionAccessor = permissionAccessor
    delete_page_permissionAccessor = permissionAccessor
    edit_page_permissionAccessor = permissionAccessor
    rename_page_permissionAccessor = permissionAccessor
    rate_page_permissionAccessor = permissionAccessor
    reparent_page_permissionAccessor = permissionAccessor
    
    
    security.declarePrivate('permissionMutator')
    def permissionMutator(self, roles, **kwargs):
        """
        Sets roles for a permission
        @param roles: List of roles with potential PSEUDO_ACQUIRED_ROLE pseudo-role
        @param kwargs: See Products/Archetypes/Field.py
        """

        roles = [role for role in roles if role] # AT Bug on widget? Empty string appended to list removed !!
        field_name = kwargs['field']
        permission = self._fieldname_to_perm.get(field_name, None)
        acquire = PSEUDO_ACQUIRED_ROLE in roles
        if acquire:
            roles.remove(PSEUDO_ACQUIRED_ROLE)
        else:
            # Manager should never get denied
            roles.append('Manager')
        self.manage_permission(permission, roles=roles, acquire=acquire)
        return
    
    # We must alias permissionMutator: AT screams when same mutator is shared by 2+ fields
    add_page_permissionMutator = permissionMutator
    add_comment_permissionMutator = permissionMutator
    change_page_type_permissionMutator = permissionMutator
    delete_page_permissionMutator = permissionMutator
    edit_page_permissionMutator = permissionMutator
    rename_page_permissionMutator = permissionMutator
    rate_page_permissionMutator = permissionMutator
    reparent_page_permissionMutator = permissionMutator

    ###
    ## Edit widget helper
    ###

    def inheritedRolesFor(self, field_name):
        """
        @param field_name: any permission related field name
        @return: inherited roles for a permission
        see skins/zwikifolder/zwikifolder_permission_widget
        """

        utranslate = self.utranslate
        permission = self._fieldname_to_perm.get(field_name, None)
        roles = rolesForPermissionOn(permission, self)
        return [utranslate(r, default=r, domain='plone')
                for r in roles]


    ###
    ## Validation
    ###

    security.declareProtected(cmf_permissions.ModifyPortalContent, 'post_validate')
    def post_validate(self, REQUEST=None, errors=None):
        """
        Validates a form.
        @see: Archetypes/BaseObject.py
        """
        
        def canValidate(field_name):
            # Discard useless validation
            return REQUEST.form.has_key(field_name) and not errors.has_key(field_name)
        
        portal_membership = getToolByName(self, 'portal_membership')
        plone_utils = getToolByName(self, 'plone_utils')
        
        if canValidate('subscriber_list'):
            invalid_names = []
            for name in REQUEST.form['subscriber_list']:
                name = name.strip()
                if name.endswith(':edits'):
                    name = name [:-6] # remove ':edits' from end
                if not (portal_membership.getMemberById(name)
                        or plone_utils.validateSingleEmailAddress(name)):
                    invalid_names.append(name)
            if invalid_names:
                message = "Not valid member ids or mail addresses: "
                message = self.utranslate('error_unvalid_members_or_emails', default=message, domain=I18N_DOMAIN)
                errors['subscriber_list'] = message + ','.join(invalid_names)
        return

at_api.registerType(ZWikiFolder)
